@@ -1,4 +1,4 @@
-#!/usr/bin/env python2
+#!/usr/bin/python22
 # -*- coding: utf-8 -*-
 
 # Copyright (c) 2015 Red Hat, Inc. <http://www.redhat.com/>
@@ -21,13 +21,13 @@ import shutil
 import tempfile
 import signal
 from datetime import datetime
+import codecs
 
 from utils import execute, is_host_local, mkdirp, fail
 from utils import setup_logger, human_time, handle_rm_error
 from utils import get_changelog_rollover_time, cache_output, create_file
 import conf
 from changelogdata import OutputMerger
-import codecs
 
 PROG_DESCRIPTION = """
 GlusterFS Incremental API
@@ -75,12 +75,27 @@ def node_cmd(host, host_uuid, task, cmd, args, opts):
             cmd = ["ssh",
                    "-oNumberOfPasswordPrompts=0",
                    "-oStrictHostKeyChecking=no",
+                   # We force TTY allocation (-t -t) so that Ctrl+C is handed
+                   # through; see:
+                   #   https://bugzilla.redhat.com/show_bug.cgi?id=1382236
+                   # Note that this turns stderr of the remote `cmd`
+                   # into stdout locally.
                    "-t",
                    "-t",
                    "-i", pem_key_path,
                    "root@%s" % host] + cmd
 
-        execute(cmd, exit_msg="%s - %s failed" % (host, task), logger=logger)
+        (returncode, err, out) = execute(cmd, logger=logger)
+        if returncode != 0:
+            # Because the `-t -t` above turns the remote stderr into
+            # local stdout, we need to log both stderr and stdout
+            # here to print all error messages.
+            fail("%s - %s failed; stdout (including remote stderr):\n"
+                 "%s\n"
+                 "stderr:\n"
+                 "%s" % (host, task, out, err),
+                 returncode,
+                 logger=logger)
 
         if opts.get("copy_outfile", False) and not localdir:
             cmd_copy = ["scp",
@@ -132,18 +147,25 @@ def run_cmd_nodes(task, args, **kwargs):
             mkdirp(os.path.dirname(node_outfile),
                    exit_on_err=True, logger=logger)
 
+            FS = args.field_separator
+            if not is_host_local(host_uuid):
+                FS = "'" + FS + "'"
+
             cmd = [change_detector,
                    args.session,
                    args.volume,
+                   host,
                    brick,
-                   node_outfile,
-                   str(kwargs.get("start"))] + \
+                   node_outfile] + \
+                ([str(kwargs.get("start")), str(kwargs.get("end"))]
+                    if not args.full else []) + \
                 ([tag] if tag is not None else []) + \
                 ["--output-prefix", args.output_prefix] + \
                 (["--debug"] if args.debug else []) + \
                 (["--no-encode"] if args.no_encode else []) + \
                 (["--only-namespace-changes"] if args.only_namespace_changes
-                 else [])
+                 else []) + \
+                (["--field-separator", FS] if args.full else [])
 
             opts["node_outfile"] = node_outfile
             opts["copy_outfile"] = True
@@ -162,19 +184,26 @@ def run_cmd_nodes(task, args, **kwargs):
             mkdirp(os.path.dirname(node_outfile),
                    exit_on_err=True, logger=logger)
 
+            FS = args.field_separator
+            if not is_host_local(host_uuid):
+                FS = "'" + FS + "'"
+
             cmd = [change_detector,
                    args.session,
                    args.volume,
+                   host,
                    brick,
-                   node_outfile,
-                   str(kwargs.get("start"))] + \
+                   node_outfile] + \
+                ([str(kwargs.get("start")), str(kwargs.get("end"))]
+                    if not args.full else []) + \
                 ([tag] if tag is not None else []) + \
                 ["--only-query"] + \
                 ["--output-prefix", args.output_prefix] + \
                 (["--debug"] if args.debug else []) + \
                 (["--no-encode"] if args.no_encode else []) + \
                 (["--only-namespace-changes"]
-                    if args.only_namespace_changes else [])
+                    if args.only_namespace_changes else []) + \
+                (["--field-separator", FS] if args.full else [])
 
             opts["node_outfile"] = node_outfile
             opts["copy_outfile"] = True
@@ -341,6 +370,8 @@ def _get_args():
                             help="Tag prefix for file names emitted during"
                             " a full find operation; default: \"NEW\"",
                             default="NEW")
+    parser_pre.add_argument("--field-separator", help="Field separator string",
+                            default=" ")
 
     # query <VOLUME> <OUTFILE> --since-time <SINCE_TIME>
     #       [--output-prefix <OUTPUT_PREFIX>] [--full]
@@ -350,6 +381,8 @@ def _get_args():
                               action=StoreAbsPath)
     parser_query.add_argument("--since-time", help="UNIX epoch time since "
                               "which listing is required", type=int)
+    parser_query.add_argument("--end-time", help="UNIX epoch time upto "
+                              "which listing is required", type=int)
     parser_query.add_argument("--no-encode",
                               help="Do not encode path in output file",
                               action="store_true")
@@ -366,6 +399,9 @@ def _get_args():
                               help="Tag prefix for file names emitted during"
                               " a full find operation; default: \"NEW\"",
                               default="NEW")
+    parser_query.add_argument("--field-separator",
+                              help="Field separator string",
+                              default=" ")
 
     # post <SESSION> <VOLUME>
     parser_post = subparsers.add_parser('post')
@@ -451,7 +487,7 @@ def enable_volume_options(args):
                 % args.volume)
 
 
-def write_output(outfile, outfilemerger):
+def write_output(outfile, outfilemerger, field_separator):
     with codecs.open(outfile, "a", encoding="utf-8") as f:
         for row in outfilemerger.get():
             # Multiple paths in case of Hardlinks
@@ -460,17 +496,22 @@ def write_output(outfile, outfilemerger):
             for p in paths:
                 if p == "":
                     continue
-                p_rep = p.replace("%2F%2F", "%2F").replace("//", "/")
+                p_rep = p.replace("//", "/")
                 if not row_2_rep:
-                    row_2_rep = row[2].replace("%2F%2F", "%2F").replace("//",
-                                                                        "/")
+                    row_2_rep = row[2].replace("//", "/")
                 if p_rep == row_2_rep:
                     continue
 
                 if row_2_rep and row_2_rep != "":
-                    f.write(u"{0} {1} {2}\n".format(row[0], p_rep, row_2_rep))
+                    f.write(u"{0}{1}{2}{3}{4}\n".format(row[0],
+                                                        field_separator,
+                                                        p_rep,
+                                                        field_separator,
+                                                        row_2_rep))
                 else:
-                    f.write(u"{0} {1}\n".format(row[0], p_rep))
+                    f.write(u"{0}{1}{2}\n".format(row[0],
+                                                  field_separator,
+                                                  p_rep))
 
 
 def mode_create(session_dir, args):
@@ -548,27 +589,37 @@ def mode_query(session_dir, args):
     enable_volume_options(args)
 
     # Test options
-    if not args.since_time and not args.full:
-        fail("Please specify either --since-time or --full", logger=logger)
+    if not args.since_time and not args.end_time and not args.full:
+        fail("Please specify either {--since-time and optionally --end-time} "
+             "or --full", logger=logger)
 
-    if args.since_time and args.full:
-        fail("Please specify either --since-time or --full, but not both",
+    if args.since_time and args.end_time and args.full:
+        fail("Please specify either {--since-time and optionally --end-time} "
+             "or --full, but not both",
              logger=logger)
 
+    if args.end_time and not args.since_time:
+        fail("Please specify --since-time as well", logger=logger)
+
     # Start query command processing
+    start = -1
+    end = -1
     if args.since_time:
         start = args.since_time
+        if args.end_time:
+            end = args.end_time
     else:
         start = 0  # --full option is handled separately
 
     logger.debug("Query is called - Session: %s, Volume: %s, "
-                 "Start time: %s"
-                 % ("default", args.volume, start))
+                 "Start time: %s, End time: %s"
+                 % ("default", args.volume, start, end))
 
     prefix = datetime.now().strftime("%Y%m%d-%H%M%S-%f-")
     gtmpfilename = prefix + next(tempfile._get_candidate_names())
 
-    run_cmd_nodes("query", args, start=start, tmpfilename=gtmpfilename)
+    run_cmd_nodes("query", args, start=start, end=end,
+                  tmpfilename=gtmpfilename)
 
     # Merger
     if args.full:
@@ -580,14 +631,14 @@ def mode_query(session_dir, args):
         # Read each Changelogs db and generate finaldb
         create_file(args.outfile, exit_on_err=True, logger=logger)
         outfilemerger = OutputMerger(args.outfile + ".db", node_outfiles)
-        write_output(args.outfile, outfilemerger)
+        write_output(args.outfile, outfilemerger, args.field_separator)
 
     try:
         os.remove(args.outfile + ".db")
     except (IOError, OSError):
         pass
 
-    run_cmd_nodes("cleanup", args)
+    run_cmd_nodes("cleanup", args, tmpfilename=gtmpfilename)
 
     sys.stdout.write("Generated output file %s\n" % args.outfile)
 
@@ -627,7 +678,7 @@ def mode_pre(session_dir, args):
     prefix = datetime.now().strftime("%Y%m%d-%H%M%S-%f-")
     gtmpfilename = prefix + next(tempfile._get_candidate_names())
 
-    run_cmd_nodes("pre", args, start=start, tmpfilename=gtmpfilename)
+    run_cmd_nodes("pre", args, start=start, end=-1, tmpfilename=gtmpfilename)
 
     # Merger
     if args.full:
@@ -639,14 +690,14 @@ def mode_pre(session_dir, args):
         # Read each Changelogs db and generate finaldb
         create_file(args.outfile, exit_on_err=True, logger=logger)
         outfilemerger = OutputMerger(args.outfile + ".db", node_outfiles)
-        write_output(args.outfile, outfilemerger)
+        write_output(args.outfile, outfilemerger, args.field_separator)
 
     try:
         os.remove(args.outfile + ".db")
     except (IOError, OSError):
         pass
 
-    run_cmd_nodes("cleanup", args)
+    run_cmd_nodes("cleanup", args, tmpfilename=gtmpfilename)
 
     with open(status_file_pre, "w", buffering=0) as f:
         f.write(str(endtime_to_update))
