@@ -1,3 +1,4 @@
+#!/usr/bin/python2
 #
 # Copyright (c) 2016 Red Hat, Inc. <http://www.redhat.com>
 # This file is part of GlusterFS.
@@ -13,3 +14,5 @@ GLUSTERD_WORKDIR = "/var/lib/glusterd"
 
 LOCALSTATEDIR = "/var"
 UUID_FILE = "/var/lib/glusterd/glusterd.info"
+GLUSTERFS_CONFDIR = "/etc/glusterfs"
+GCONF_VERSION = 4.0
